@@ -15,7 +15,7 @@ my ( $mode, $master, %slaves, %trusted_slaves );
 # ----------------------------------------------------------------------
 
 sub input {
-    unless ( $ARGV[0] =~ /^server-(\S+)$/ ) {
+    unless ($ARGV[0] =~ /^server-(\S+)$/) {
         _die "'$ARGV[0]' is not a valid server name" if $ENV{SSH_ORIGINAL_COMMAND} =~ /^USER=(\S+) SOC=(git-receive-pack '(\S+)')$/;
         return;
     }
@@ -27,19 +27,17 @@ sub input {
 
     # custom peer-to-peer commands.  At present the only one is 'perms -c',
     # sent from a mirror command
-    if ( $ENV{SSH_ORIGINAL_COMMAND} =~ /^CREATOR=(\S+) perms -c '(\S+)'$/ ) {
+    if ($ENV{SSH_ORIGINAL_COMMAND} =~ /^CREATOR=(\S+) perms -c '(\S+)'$/) {
         $ENV{GL_USER} = $1;
 
         my $repo = $2;
         details($repo);
-        _die "$hn: '$repo' is local"                        if $mode eq 'local';
-        _die "$hn: '$repo' is native"                       if $mode eq 'master';
+        _die "$hn: '$repo' is local"  if $mode eq 'local';
+        _die "$hn: '$repo' is native" if $mode eq 'master';
         _die "$hn: '$sender' is not the master for '$repo'" if $master ne $sender;
 
-        $ENV{GL_BYPASS_CREATOR_CHECK} = option($repo, "bypass-creator-check");
         # this expects valid perms content on STDIN
         _system("gitolite perms -c $repo");
-        delete $ENV{GL_BYPASS_CREATOR_CHECK};
 
         # we're done.  Yes, really...
         exit 0;
@@ -62,7 +60,7 @@ sub input {
 sub pre_git {
     return unless $hn;
     # nothing, and I mean NOTHING, happens if HOSTNAME is not set
-    trace( 3, "pre_git() on $hn" );
+    trace( 1, "pre_git() on $hn" );
 
     my ( $repo, $user, $aa ) = @_[ 1, 2, 3 ];
 
@@ -73,9 +71,6 @@ sub pre_git {
     # now you know the repo, get its mirroring details
     details($repo);
 
-    # print mirror status if at least one slave status file is present
-    print_status( $repo ) if not $rc{HUSH_MIRROR_STATUS} and $mode ne 'local' and glob("$rc{GL_REPO_BASE}/$repo.git/gl-slave-*.status");
-
     # we don't deal with any reads.  Note that for pre-git this check must
     # happen *after* getting details, to give mode() a chance to die on "known
     # unknown" repos (repos that are in the config, but mirror settings
@@ -90,6 +85,7 @@ sub pre_git {
         trace( 3, "case 1, user push" );
         return if $mode eq 'local' or $mode eq 'master';
         if ( $trusted_slaves{$hn} ) {
+            trace( 3, "redirecting to $master" );
             trace( 1, "redirect to $master" );
             exec( "ssh", $master, "USER=$user", "SOC=$ENV{SSH_ORIGINAL_COMMAND}" );
         } else {
@@ -101,8 +97,8 @@ sub pre_git {
     # case 2: we're slave, master pushing to us
     if ( $sender and not $rc{REDIRECTED_PUSH} ) {
         trace( 3, "case 2, master push" );
-        _die "$hn: '$repo' is local"                        if $mode eq 'local';
-        _die "$hn: '$repo' is native"                       if $mode eq 'master';
+        _die "$hn: '$repo' is local"  if $mode eq 'local';
+        _die "$hn: '$repo' is native" if $mode eq 'master';
         _die "$hn: '$sender' is not the master for '$repo'" if $master ne $sender;
         return;
     }
@@ -111,8 +107,8 @@ sub pre_git {
     # case 3: we're master, slave sending a redirected push to us
     if ( $sender and $rc{REDIRECTED_PUSH} ) {
         trace( 3, "case 2, slave redirect" );
-        _die "$hn: '$repo' is local"                           if $mode eq 'local';
-        _die "$hn: '$repo' is not native"                      if $mode eq 'slave';
+        _die "$hn: '$repo' is local"      if $mode eq 'local';
+        _die "$hn: '$repo' is not native" if $mode eq 'slave';
         _die "$hn: '$sender' is not a valid slave for '$repo'" if not $slaves{$sender};
         _die "$hn: redirection not allowed from '$sender'"     if not $trusted_slaves{$sender};
         return;
@@ -194,17 +190,8 @@ sub post_git {
     }
 
     sub slaves {
-        my $repo = shift;
-
-        my $ref = git_config( $repo, "^gitolite-options\\.mirror\\.slaves.*" );
-        my %out = map { $_ => 'async' } map { split } values %$ref;
-
-        $ref = git_config( $repo, "^gitolite-options\\.mirror\\.slaves\\.sync.*" );
-        map { $out{$_} = 'sync' } map { split } values %$ref;
-
-        $ref = git_config( $repo, "^gitolite-options\\.mirror\\.slaves\\.nosync.*" );
-        map { $out{$_} = 'nosync' } map { split } values %$ref;
-
+        my $ref = git_config( +shift, "^gitolite-options\\.mirror\\.slaves.*" );
+        my %out = map { $_ => 1 } map { split } values %$ref;
         return %out;
     }
 
@@ -235,23 +222,11 @@ sub push_to_slaves {
     my $u = $ENV{GL_USER};
     delete $ENV{GL_USER};    # why?  see src/commands/mirror
 
-    my $lb = "$ENV{GL_REPO_BASE}/$repo.git/.gl-mirror-lock";
     for my $s ( sort keys %slaves ) {
-        trace( 1, "push_to_slaves: skipping self" ), next if $s eq $hn;
-        system("gitolite 1plus1 $lb.$s gitolite mirror push $s $repo </dev/null >/dev/null 2>&1 &") if $slaves{$s} eq 'async';
-        system("gitolite 1plus1 $lb.$s gitolite mirror push $s $repo </dev/null >/dev/null 2>&1")   if $slaves{$s} eq 'sync';
-        _warn "manual mirror push pending for '$s'"                          if $slaves{$s} eq 'nosync';
+        system("gitolite mirror push $s $repo &");
     }
 
     $ENV{GL_USER} = $u;
 }
 
-sub print_status {
-    my $repo = shift;
-    my $u = $ENV{GL_USER};
-    delete $ENV{GL_USER};
-    system("gitolite mirror status all $repo >&2");
-    $ENV{GL_USER} = $u;
-}
-
 1;
